
// gillstep generated Markov model code

#ifndef __VOLTAGEGATE_H
#define __VOLTAGEGATE_H

#include <vector>
#include <map>
#include <cmath>
#include <cassert>
#include <stdexcept>
#include <algorithm>
#include <boost/bind.hpp>

#include <gillstep/common.h>
#include <gillstep/MarkovModel.h>
#include <gillstep/GillespieStepper.h>
#include <gillstep/temperature.h>

namespace gillstep
{

  /// Markov model 
  class VoltageGate : public MarkovModel
  {
      
  
  public:

    /// Default constructor
    VoltageGate(GillespieStepper& gillstepper) :
        MarkovModel("VoltageGate", gillstepper),
        _retabulate_derivatives(true),
        _present_substep(0.0),
        _variables_05(1),
        _rate_05(1),
        _rate_deriviatives_05(1)
        
    {
      /// Resize field
      _variables_0.resize(1);
      _variables_1.resize(1);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;

      _value_receivers["V"] = boost::bind(&VoltageGate::_receive_V, this, _1);
      _variable_map["V"] = 0;
    }
    
    /// SubMarkov model constructor
    VoltageGate(MarkovModel* parent, GillespieStepper& gillstepper) :
        MarkovModel("VoltageGate", gillstepper),
        _retabulate_derivatives(true),
        _present_substep(0.0),
        _variables_05(1),
        _rate_05(1),
        _rate_deriviatives_05(1)
    {
      // Assign parent pointer
      _parent = parent;

      /// Resize field
      _variables_0.resize(1);
      _variables_1.resize(1);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;

      _value_receivers["V"] = boost::bind(&VoltageGate::_receive_V, this, _1);
      _variable_map["V"] = 0;

      // Check if model can receive "parent"
      if (can_receive(parent->name()))
      {
        parent->connect(receive(parent->name()));
        _receive_parent = true;
      }
    
    }
    
    /// Destructor
    virtual ~VoltageGate(){};
    
    /// Returns the maximal transitions from a state
    uint max_transitions() const
    {
       return 1;
    }

    /// Return the 'value' of the Markov model the state represent 
    double value() const
    {
      switch (_state) {
        case 1:
          return 1;
      default:
        return 0.0;
      }
    
    }

    /// Return the size of the substep. 1.0 if no substep is needed
    double substep_size(double present_substep)
    {
      assert(present_substep >= 0.0);
        
      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep){
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      double next_substep = 1;
      for (uint trans = 0; trans < _transition_map[_state].size(); trans++){
        next_substep = std::min(next_substep, std::fabs(
                       _gillstepper->get_max_fractional_rate_change()*
                       (_rate_05[trans]/_rate_deriviatives_05[trans])));
      }
      // Update the the next substep result with the present one
      next_substep = present_substep + next_substep;

      // Return next substep. Return 1.0 if it is larger than 1.0
      return next_substep > 1.0 ? 1.0 : next_substep;
    }

    /// Evaluate rate function corresponding to passed transition
    double evaluate_rate(uint transition) const
    {
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=VoltageGate::_transition_map[_state].size())
        return 0.0;

      // Evaluate the rate
      return VoltageGate::_rate_functions[_state][transition](*this);
    }
    
    /// Evaluate rate function corresponding to passed state and transition
    double evaluate_rate(uint from_state, uint to_state) const
    {
      switch (from_state) {
        case 0:
          switch (to_state) {
            case 1:
              return VoltageGate::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
        case 1:
          switch (to_state) {
            case 0:
              return VoltageGate::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
      default:
        return 0.0;
      }
    }
    
    /// Evaluate rate function corresponding to passed transition for a substep
    double evaluate_substep_rate(uint transition, double present_substep,
                                 double next_substep)
    {
      assert(present_substep>=0);
      assert(next_substep>=present_substep);
      
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=VoltageGate::_transition_map[_state].size())
        return 0.0;

      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep){
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      // Use tabulated values
      return _rate_05[transition] + (next_substep-_present_substep)*
        (_rate_deriviatives_05[transition]);
    }
    
    /// Change the state to the given transition
    bool change_state(int transition)
    {
      /// If forced to not change the value of this Markov Model
      if (fix_state)
        return false;
        
      /// We need to retabulate derivatives
      _retabulate_derivatives = true;
        
      const int num_transitions = VoltageGate::_transition_map[_state].size();
      assert(transition<num_transitions);
        
      /// Update the state from the transition
      if (transition>=0)
        _state = VoltageGate::_transition_map[_state][transition];

      /// If this is a SubMarkov model return the parents change_state value
      if (_parent)
        return _parent->change_state(-1);

      /// If this is not a SubMarkov model check if my value has change
      const bool changed = _old_value != value();
      _old_value = value();
      return changed;
    }

    /// Get temperature scale
    static inline double temp_scale()
    {
      return VoltageGate::_temp_scale;
    }

    /// Get parameter
    static inline double get_parameter(uint ind)
    {
      return VoltageGate::_params[ind];
    }

    /// Get parameter
    double get_parameter(const std::string& name) const
    {
      return _params[_find_parameter(name)];
    }
    
    /// Set parameter
    void set_parameter(const std::string& name, double value)
    {
      _params[_find_parameter(name)] = value;
      // If the temperature scale is change update it
      if (name == "T_ref" or name == "Q10")
        update_temp_scale();
    }

    /// Get parameter names
    virtual std::vector<std::string> get_parameter_names() const
    {
      std::vector<std::string> parameter_names;

      parameter_names.push_back("Q10");
      parameter_names.push_back("T_ref");

      // Return by value
      return parameter_names;
    };

    /// Return the number of states
    inline uint num_states() const
    {
      return 2;
    }

    /// Update the temperature scale factor
    void update_temp_scale()
    {
      VoltageGate::_temp_scale = pow(VoltageGate::_params[0], ((get_T()-T_abs())-VoltageGate::_params[1])/10);
    }

  private:

    /// Tabulate rate derivatives
    void _tabulate_rate_deriviatives()
    {
      // Local variables
      uint trans, var;

      // Update variables at present substep
      for (var=0; var < _variables_05.size(); var++)
        _variables_05[var] = _variables_1[var]*_present_substep +
                             (1-_present_substep)*_variables_0[var];
        
      // Set the present variables to present interpolated values
      _present_variables = &_variables_05;

      for (trans=0; trans < _transition_map[_state].size(); trans++){
        _rate_05[trans] = VoltageGate::_rate_functions[_state][trans](*this);
        _rate_deriviatives_05[trans] = 0.0;
        for (var=0; var<_variables_0.size(); var++){
          _rate_deriviatives_05[trans] += VoltageGate::_rate_derivative_functions
                                         [_state][trans][var](*this)*
                                         (_variables_1[var] - _variables_05[var]);
        }
      }

      // Set the present variables to the last received values
      _present_variables = &_variables_1;

      // Set flag preventing unessesary tabulation
      _retabulate_derivatives = false;
    }

    /// Check parameter name
    static uint _find_parameter(const std::string& name)                              
    {
      std::map<std::string, uint>::const_iterator it = VoltageGate::_param_map.find(name);
      if (it == VoltageGate::_param_map.end())
        throw std::runtime_error("\"" + name + "\" is not a parameter in this Markov Model");                         
      return it->second;
    }

    
    /// Receive methods
    inline void _receive_V(double value)
    {
      _retabulate_derivatives = true;
      _variables_0[0] = _variables_1[0];
      _variables_1[0] = value;
    }
    

    /// Data members

    /// Variable difference
    bool _retabulate_derivatives;

    // Keep track of which substep we are on
    double _present_substep;

    // Variables values inside time step
    std::vector<double> _variables_05;

    /// The interpolated rate values and its derivatives wrt to the variables
    std::vector<double> _rate_05;
    std::vector<double> _rate_deriviatives_05;

      
    /// Temperature scale factor
    static double _temp_scale;

    /// An array for the parameter values
    static std::vector<double> _params;

    /// Mapping names to parameter indices
    static const std::map<std::string, uint> _param_map;
    
    /// Mapping state and transition number to state
    static const std::vector<std::vector<uint> > _transition_map;

    /// Mapping state and transition number to rate function
    static const std::vector<std::vector<rate_call> > _rate_functions;

    /// Mapping state, transition number and continuous variables to rate
    /// derivative function
    static const std::vector<std::vector<std::vector<rate_call> > > _rate_derivative_functions;
    
  };

  // Name space for hiding static initalization functions
  namespace InitVoltageGate
  {

    /// Define Rate functions
    
    /// Rate function between state: 0 and 1 
    double rate_func_0_1(const MarkovModel& mm)
    {
      return VoltageGate::temp_scale()*((0.6 + 0.4/(1 + exp(12.5 + mm._unsafe_variable(0))/5))/(60 + 340/(1 + exp(30 + mm._unsafe_variable(0))/12)));
    }
    
    
    /// Rate function between state: 1 and 0 
    double rate_func_1_0(const MarkovModel& mm)
    {
      return VoltageGate::temp_scale()*((0.4 - 0.4/(1 + exp(12.5 + mm._unsafe_variable(0))/5))/(60 + 340/(1 + exp(30 + mm._unsafe_variable(0))/12)));
    }
    
    

    /// Define derivatives of Rate functions
    
    /// Derivative of rate function between state: 0 and 1 for variable: V
    double der_rate_func_0_1_wrt_V(const MarkovModel& mm)
    {
      return VoltageGate::temp_scale()*(85*(0.6 + 0.4/(1 + exp(12.5 + mm._unsafe_variable(0))/5))*exp(30 + mm._unsafe_variable(0))/(3*pow((1 + exp(30 + mm._unsafe_variable(0))/12),2)*pow((60 + 340/(1 + exp(30 + mm._unsafe_variable(0))/12)),2)) - 0.08*exp(12.5 + mm._unsafe_variable(0))/(pow((1 + exp(12.5 + mm._unsafe_variable(0))/5),2)*(60 + 340/(1 + exp(30 + mm._unsafe_variable(0))/12))));
    }
    
    
    /// Derivative of rate function between state: 1 and 0 for variable: V
    double der_rate_func_1_0_wrt_V(const MarkovModel& mm)
    {
      return VoltageGate::temp_scale()*(85*(0.4 - 0.4/(1 + exp(12.5 + mm._unsafe_variable(0))/5))*exp(30 + mm._unsafe_variable(0))/(3*pow((1 + exp(30 + mm._unsafe_variable(0))/12),2)*pow((60 + 340/(1 + exp(30 + mm._unsafe_variable(0))/12)),2)) + 0.08*exp(12.5 + mm._unsafe_variable(0))/(pow((1 + exp(12.5 + mm._unsafe_variable(0))/5),2)*(60 + 340/(1 + exp(30 + mm._unsafe_variable(0))/12))));
    }
    
    
    /// Init params
    std::vector<double> params()
    {
      // vector which will be returned by value
      std::vector<double> params_;
      params_.push_back(1.000000);
      params_.push_back(37.000000);

      // return by value
      return params_;
    }

    /// Init param_map
    const std::map<std::string, uint> param_map()
    {
      // map which will be returned by value
      std::map<std::string, uint> param_map_;
      param_map_["Q10"] = 0;
      param_map_["T_ref"] = 1;

      // return by value
      return param_map_;
    }

    /// Init transition_map
    const std::vector<std::vector<uint> > transition_map()
    {
      // vector which will be returned by value
      std::vector<std::vector<uint> > transition_map_;
      
      // Transition map from state 0
      std::vector<uint> transition_map_0;
      transition_map_0.push_back(1);
      transition_map_.push_back(transition_map_0);
      
      // Transition map from state 1
      std::vector<uint> transition_map_1;
      transition_map_1.push_back(0);
      transition_map_.push_back(transition_map_1);

      // return by value
      return transition_map_;
    }

    /// Init rate_functions
    const std::vector<std::vector<rate_call> > rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<rate_call> > rate_functions_;
      
      // Rate functions from state 0
      std::vector<rate_call> rate_functions_0;
      rate_functions_0.push_back(rate_func_0_1);
      rate_functions_.push_back(rate_functions_0);
      
      // Rate functions from state 1
      std::vector<rate_call> rate_functions_1;
      rate_functions_1.push_back(rate_func_1_0);
      rate_functions_.push_back(rate_functions_1);

      // return by value
      return rate_functions_;
    }
    
    /// Init derivative of rate_functions
    const std::vector<std::vector<std::vector<rate_call> > > der_rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<std::vector<rate_call> > > der_rate_functions_;
      
      // Derivative of rate functions from state 0
      std::vector<std::vector<rate_call> > der_rate_functions_0;
      
      // Derivative of rate functions from state 0 to state 1
      std::vector<rate_call> der_rate_functions_0_1;
      der_rate_functions_0_1.push_back(der_rate_func_0_1_wrt_V);
      der_rate_functions_0.push_back(der_rate_functions_0_1);
      
      der_rate_functions_.push_back(der_rate_functions_0);
      
      // Derivative of rate functions from state 1
      std::vector<std::vector<rate_call> > der_rate_functions_1;
      
      // Derivative of rate functions from state 1 to state 0
      std::vector<rate_call> der_rate_functions_1_0;
      der_rate_functions_1_0.push_back(der_rate_func_1_0_wrt_V);
      der_rate_functions_1.push_back(der_rate_functions_1_0);
      
      der_rate_functions_.push_back(der_rate_functions_1);

      // return by value
      return der_rate_functions_;
    }
    
  }

  /// Initialize static variables
  double VoltageGate::_temp_scale = 1.0;
  std::vector<double> VoltageGate::_params = InitVoltageGate::params();
  const std::map<std::string, uint> VoltageGate::_param_map = InitVoltageGate::param_map();
  const std::vector<std::vector<uint> > VoltageGate::_transition_map = InitVoltageGate::transition_map();
  const std::vector<std::vector<rate_call> > VoltageGate::_rate_functions = InitVoltageGate::rate_functions();
  const std::vector<std::vector<std::vector<rate_call> > > VoltageGate::_rate_derivative_functions = InitVoltageGate::der_rate_functions();

}

#endif
