# A complex test Markokov model of four states
#
#      0  -  1  -  2  -  3
#

from gillstep import *

complex_model = ModelEditor("Complex")

# Declarations Note that parameters are returned in a sorted order
complex_model.set_parameters(
    alpha_0 = 1.0, alpha_1 = 2.0,
    beta_0  = 3.0, beta_1  = 4.0,
    gamma_0 = 5.0, a       = 6.0)

complex_model.set_variables("V", "dyadic_Ca")

complex_model.set_Q10(1.8, 37)

# Rate functions
alpha = exp(-(20-V/(R*T))/10)
beta  = exp(-(V-35)/3)
gamma = gamma_0*dyadic_Ca

# Load sub models
voltage_gate = load_submodel("VoltageGate")
ca_bind1, ca_bind2, ca_bind3, ca_bind4  = \
       load_submodel("Ca_bind", 4)

# Assign the rates
rates = complex_model.rates
rates[0,1] = 2*alpha_0*alpha + 2
rates[1,2] = alpha_1*alpha
rates[1,0] = beta_0*beta
rates[2,1] = 2*beta_1*beta
rates[3,2] = gamma*(ca_bind1.value + ca_bind2.value + \
                    ca_bind3.value + ca_bind4.value)
rates[2,3] = a**2

# State values
complex_model.state_values[3] = voltage_gate.value

# Instantiate the GillespieStepper
gillstepper = GillespieStepper()

# JIT compile the markov model
mm, mm2 = jit(complex_model, gillstepper, 2)

mm.receive("V")(-5.0)
mm.receive("dyadic_Ca")(1.0)
for sm in mm.submodels("VoltageGate"):
    sm.receive("V")(-5.0)

for sm in mm.submodels("Ca_bind"):
    sm.receive("dyadic_Ca")(1.0)

sm = mm.submodel(0)
sm.change_state(0)
print "changed state, value changed?", mm.change_state(0), "new state:", mm.state() 
print "changed state, value changed?", mm.change_state(1), "new state:", mm.state()

print mm.change_state(1)

print "changed state, value changed?", mm.change_state(1), "new state:", mm.state() 

trans = 0

print "Value:", sm.value()
print "Value, sub:", mm2.value()
print "state:", mm.state()
print "Value:", mm.value()
print "T_ref:", mm.get_parameter("T_ref")
print "alpha_0:", mm.get_parameter("alpha_0")
print "gamma_0:", mm.get_parameter("gamma_0")
print "rate ", mm.state(), trans, ":", mm.evaluate_rate(trans)

trans = 1
print "changed state, value changed? ", mm.change_state(0), "new state:", mm.state()
print "rate ", mm.state(), trans, ":", mm.evaluate_rate(trans)
mm.set_parameter("a", 10)
print "rate ", mm.state(), trans, ":", mm.evaluate_rate(trans)

print "T:", get_T()-T_abs()
set_T(22)
print "T:", get_T()-T_abs()
mm.update_temp_scale()
print "rate ", mm.state(), trans, ":", mm.evaluate_rate(trans)

print "Num MarkovModels:", gillstepper.num_markov_models()
