#include <iostream>
#include <vector>
#include <utility>

#include <gillstep.h>
#include "LCC.h"
#include "gnuplot-iostream.h"

using namespace gillstep;
using gillstep::uint;

const uint N_LCC = 2000;

void plot_data(const DiscreteValueReceiver& values)
{
  
  const std::vector<double> times = values.get_times();
  const std::vector<uint> num_open = values.get_num_open();
  std::vector<std::pair<double, double> > num_open_map;
  
  for (uint i=0; i < num_open.size(); i++)
    num_open_map.push_back(std::make_pair(times[i], static_cast<double>(num_open[i]) / N_LCC));

  Gnuplot gp;
  gp << "set terminal postscript eps color \n";
  gp << "set output 'LCC_open_stats_N_LCC_" << N_LCC << ".eps'\n";
  gp << "p '-' w l\n";
  gp.send(num_open_map);
}

int main()
{

  GillespieStepper gillstepper;
  
  std::vector<LCC*> lcc_vec;

  DiscreteValueStepper voltage;
  DiscreteValueStepper calcium;
  
  DiscreteValueReceiver values(gillstepper);
  
  for (uint i = 0; i < N_LCC; i++)
  {
    LCC* lcc = new LCC(gillstepper);
    voltage.connect(lcc->receive("V"));
    calcium.connect(lcc->receive("dyadic_Ca"));
    lcc->connect(values.receive);
    lcc_vec.push_back(lcc);
  }
  
  // Inital values
  voltage.send(-85.0);
  calcium.send(.1);

  // Reset LCCs
  gillstepper.reset();

  voltage.send(20.0);

  // Step the Markov models for 5.0 ms
  gillstepper.step_dt(5.0);
  voltage.send(-85.0);
  gillstepper.step_dt(5.0);
  voltage.send(20.0);
  gillstepper.step_dt(5.0);

  // Use Gnu plot to plot the data
  //plot_data(values);

  for (uint i = 0; i < N_LCC; i++)
    delete lcc_vec[i];

  return 0;

}
