
// gillstep generated Markov model code

#ifndef __SIMPLE_H
#define __SIMPLE_H

#include <vector>
#include <map>
#include <cmath>
#include <cassert>
#include <stdexcept>
#include <algorithm>
#include <boost/bind.hpp>

#include <gillstep/common.h>
#include <gillstep/MarkovModel.h>
#include <gillstep/GillespieStepper.h>
#include <gillstep/temperature.h>

namespace gillstep
{

  /// Markov model 
  class simple : public MarkovModel
  {
      
  
  public:

    /// Default constructor
    simple(GillespieStepper& gillstepper) :
        MarkovModel("simple", gillstepper),
        _retabulate_derivatives(true),
        _present_substep(0.0),
        _variables_05(2),
        _rate_05(2),
        _rate_deriviatives_05(2)
        
    {
      /// Resize field
      _variables_0.resize(2);
      _variables_1.resize(2);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;

      _value_receivers["V"] = boost::bind(&simple::_receive_V, this, _1);
      _variable_map["V"] = 0;
      _value_receivers["dyadic_Ca"] = boost::bind(&simple::_receive_dyadic_Ca, this, _1);
      _variable_map["dyadic_Ca"] = 1;
    }
    
    /// SubMarkov model constructor
    simple(MarkovModel* parent, GillespieStepper& gillstepper) :
        MarkovModel("simple", gillstepper),
        _retabulate_derivatives(true),
        _present_substep(0.0),
        _variables_05(2),
        _rate_05(2),
        _rate_deriviatives_05(2)
    {
      // Assign parent pointer
      _parent = parent;

      /// Resize field
      _variables_0.resize(2);
      _variables_1.resize(2);
      _discrete_variables.resize(0);
      _present_variables = &_variables_1;

      _value_receivers["V"] = boost::bind(&simple::_receive_V, this, _1);
      _variable_map["V"] = 0;
      _value_receivers["dyadic_Ca"] = boost::bind(&simple::_receive_dyadic_Ca, this, _1);
      _variable_map["dyadic_Ca"] = 1;

      // Check if model can receive "parent"
      if (can_receive(parent->name()))
      {
        parent->connect(receive(parent->name()));
        _receive_parent = true;
      }
    
    }
    
    /// Destructor
    virtual ~simple(){};
    
    /// Returns the maximal transitions from a state
    uint max_transitions() const
    {
       return 2;
    }

    /// Return the 'value' of the Markov model the state represent 
    double value() const
    {
      switch (_state) {
        case 3:
          return 1;
      default:
        return 0.0;
      }
    
    }

    /// Return the size of the substep. 1.0 if no substep is needed
    double substep_size(double present_substep)
    {
      assert(present_substep >= 0.0);
        
      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep){
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      double next_substep = 1;
      for (uint trans = 0; trans < _transition_map[_state].size(); trans++){
        next_substep = std::min(next_substep, std::fabs(
                       _gillstepper->get_max_fractional_rate_change()*
                       (_rate_05[trans]/_rate_deriviatives_05[trans])));
      }
      // Update the the next substep result with the present one
      next_substep = present_substep + next_substep;

      // Return next substep. Return 1.0 if it is larger than 1.0
      return next_substep > 1.0 ? 1.0 : next_substep;
    }

    /// Evaluate rate function corresponding to passed transition
    double evaluate_rate(uint transition) const
    {
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=simple::_transition_map[_state].size())
        return 0.0;

      // Evaluate the rate
      return simple::_rate_functions[_state][transition](*this);
    }
    
    /// Evaluate rate function corresponding to passed state and transition
    double evaluate_rate(uint from_state, uint to_state) const
    {
      switch (from_state) {
        case 0:
          switch (to_state) {
            case 1:
              return simple::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
        case 1:
          switch (to_state) {
            case 0:
              return simple::_rate_functions[from_state][0](*this);
            case 2:
              return simple::_rate_functions[from_state][1](*this);
        default:
          return 0.0;
        }
        case 2:
          switch (to_state) {
            case 1:
              return simple::_rate_functions[from_state][0](*this);
            case 3:
              return simple::_rate_functions[from_state][1](*this);
        default:
          return 0.0;
        }
        case 3:
          switch (to_state) {
            case 2:
              return simple::_rate_functions[from_state][0](*this);
        default:
          return 0.0;
        }
      default:
        return 0.0;
      }
    }
    
    /// Evaluate rate function corresponding to passed transition for a substep
    double evaluate_substep_rate(uint transition, double present_substep,
                                 double next_substep)
    {
      assert(present_substep>=0);
      assert(next_substep>=present_substep);
      
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=simple::_transition_map[_state].size())
        return 0.0;

      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep){
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      // Use tabulated values
      return _rate_05[transition] + (next_substep-_present_substep)*
        (_rate_deriviatives_05[transition]);
    }
    
    /// Change the state to the given transition
    bool change_state(int transition)
    {
      /// If forced to not change the value of this Markov Model
      if (fix_state)
        return false;
        
      /// We need to retabulate derivatives
      _retabulate_derivatives = true;
        
      const int num_transitions = simple::_transition_map[_state].size();
      assert(transition<num_transitions);
        
      /// Update the state from the transition
      if (transition>=0)
        _state = simple::_transition_map[_state][transition];

      /// If this is a SubMarkov model return the parents change_state value
      if (_parent)
        return _parent->change_state(-1);

      /// If this is not a SubMarkov model check if my value has change
      const bool changed = _old_value != value();
      _old_value = value();
      return changed;
    }

    /// Get temperature scale
    static inline double temp_scale()
    {
      return simple::_temp_scale;
    }

    /// Get parameter
    static inline double get_parameter(uint ind)
    {
      return simple::_params[ind];
    }

    /// Get parameter
    double get_parameter(const std::string& name) const
    {
      return _params[_find_parameter(name)];
    }
    
    /// Set parameter
    void set_parameter(const std::string& name, double value)
    {
      _params[_find_parameter(name)] = value;
      // If the temperature scale is change update it
      if (name == "T_ref" or name == "Q10")
        update_temp_scale();
    }

    /// Get parameter names
    virtual std::vector<std::string> get_parameter_names() const
    {
      std::vector<std::string> parameter_names;

      parameter_names.push_back("Q10");
      parameter_names.push_back("T_ref");
      parameter_names.push_back("a");
      parameter_names.push_back("alpha_0");
      parameter_names.push_back("alpha_1");
      parameter_names.push_back("beta_0");
      parameter_names.push_back("beta_1");
      parameter_names.push_back("gamma_0");

      // Return by value
      return parameter_names;
    };

    /// Return the number of states
    inline uint num_states() const
    {
      return 4;
    }

    /// Update the temperature scale factor
    void update_temp_scale()
    {
      simple::_temp_scale = pow(simple::_params[0], ((get_T()-T_abs())-simple::_params[1])/10);
    }

  private:

    /// Tabulate rate derivatives
    void _tabulate_rate_deriviatives()
    {
      // Local variables
      uint trans, var;

      // Update variables at present substep
      for (var=0; var < _variables_05.size(); var++)
        _variables_05[var] = _variables_1[var]*_present_substep +
                             (1-_present_substep)*_variables_0[var];
        
      // Set the present variables to present interpolated values
      _present_variables = &_variables_05;

      for (trans=0; trans < _transition_map[_state].size(); trans++){
        _rate_05[trans] = simple::_rate_functions[_state][trans](*this);
        _rate_deriviatives_05[trans] = 0.0;
        for (var=0; var<_variables_0.size(); var++){
          _rate_deriviatives_05[trans] += simple::_rate_derivative_functions
                                         [_state][trans][var](*this)*
                                         (_variables_1[var] - _variables_05[var]);
        }
      }

      // Set the present variables to the last received values
      _present_variables = &_variables_1;

      // Set flag preventing unessesary tabulation
      _retabulate_derivatives = false;
    }

    /// Check parameter name
    static uint _find_parameter(const std::string& name)                              
    {
      std::map<std::string, uint>::const_iterator it = simple::_param_map.find(name);
      if (it == simple::_param_map.end())
        throw std::runtime_error("\"" + name + "\" is not a parameter in this Markov Model");                         
      return it->second;
    }

    
    /// Receive methods
    inline void _receive_V(double value)
    {
      _retabulate_derivatives = true;
      _variables_0[0] = _variables_1[0];
      _variables_1[0] = value;
    }
    
    inline void _receive_dyadic_Ca(double value)
    {
      _retabulate_derivatives = true;
      _variables_0[1] = _variables_1[1];
      _variables_1[1] = value;
    }
    

    /// Data members

    /// Variable difference
    bool _retabulate_derivatives;

    // Keep track of which substep we are on
    double _present_substep;

    // Variables values inside time step
    std::vector<double> _variables_05;

    /// The interpolated rate values and its derivatives wrt to the variables
    std::vector<double> _rate_05;
    std::vector<double> _rate_deriviatives_05;

      
    /// Temperature scale factor
    static double _temp_scale;

    /// An array for the parameter values
    static std::vector<double> _params;

    /// Mapping names to parameter indices
    static const std::map<std::string, uint> _param_map;
    
    /// Mapping state and transition number to state
    static const std::vector<std::vector<uint> > _transition_map;

    /// Mapping state and transition number to rate function
    static const std::vector<std::vector<rate_call> > _rate_functions;

    /// Mapping state, transition number and continuous variables to rate
    /// derivative function
    static const std::vector<std::vector<std::vector<rate_call> > > _rate_derivative_functions;
    
  };

  // Name space for hiding static initalization functions
  namespace Initsimple
  {

    /// Define Rate functions
    
    /// Rate function between state: 0 and 1 
    double rate_func_0_1(const MarkovModel& mm)
    {
      return simple::temp_scale()*(2*simple::get_parameter(3)*exp(-2 + 0.0120272218305736*mm._unsafe_variable(0)/get_T()));
    }
    
    
    /// Rate function between state: 1 and 2 
    double rate_func_1_2(const MarkovModel& mm)
    {
      return simple::temp_scale()*(simple::get_parameter(4)*exp(-2 + 0.0120272218305736*mm._unsafe_variable(0)/get_T()));
    }
    
    
    /// Rate function between state: 3 and 2 
    double rate_func_3_2(const MarkovModel& mm)
    {
      return simple::temp_scale()*(mm._unsafe_variable(1)*simple::get_parameter(7));
    }
    
    
    /// Rate function between state: 2 and 1 
    double rate_func_2_1(const MarkovModel& mm)
    {
      return simple::temp_scale()*(2*simple::get_parameter(6)*exp(35.0/3.0 - mm._unsafe_variable(0)/3));
    }
    
    
    /// Rate function between state: 2 and 3 
    double rate_func_2_3(const MarkovModel& mm)
    {
      return simple::temp_scale()*(pow(simple::get_parameter(2),2));
    }
    
    
    /// Rate function between state: 1 and 0 
    double rate_func_1_0(const MarkovModel& mm)
    {
      return simple::temp_scale()*(simple::get_parameter(5)*exp(35.0/3.0 - mm._unsafe_variable(0)/3));
    }
    
    

    /// Define derivatives of Rate functions
    
    /// Derivative of rate function between state: 0 and 1 for variable: V
    double der_rate_func_0_1_wrt_V(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0.0240544436611471*simple::get_parameter(3)*exp(-2 + 0.0120272218305736*mm._unsafe_variable(0)/get_T())/get_T());
    }
    
    
    /// Derivative of rate function between state: 0 and 1 for variable: dyadic_Ca
    double der_rate_func_0_1_wrt_dyadic_Ca(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0);
    }
    
    
    /// Derivative of rate function between state: 1 and 2 for variable: V
    double der_rate_func_1_2_wrt_V(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0.0120272218305736*simple::get_parameter(4)*exp(-2 + 0.0120272218305736*mm._unsafe_variable(0)/get_T())/get_T());
    }
    
    
    /// Derivative of rate function between state: 1 and 2 for variable: dyadic_Ca
    double der_rate_func_1_2_wrt_dyadic_Ca(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0);
    }
    
    
    /// Derivative of rate function between state: 3 and 2 for variable: V
    double der_rate_func_3_2_wrt_V(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0);
    }
    
    
    /// Derivative of rate function between state: 3 and 2 for variable: dyadic_Ca
    double der_rate_func_3_2_wrt_dyadic_Ca(const MarkovModel& mm)
    {
      return simple::temp_scale()*(simple::get_parameter(7));
    }
    
    
    /// Derivative of rate function between state: 2 and 1 for variable: V
    double der_rate_func_2_1_wrt_V(const MarkovModel& mm)
    {
      return simple::temp_scale()*(-2*simple::get_parameter(6)*exp(35.0/3.0 - mm._unsafe_variable(0)/3)/3);
    }
    
    
    /// Derivative of rate function between state: 2 and 1 for variable: dyadic_Ca
    double der_rate_func_2_1_wrt_dyadic_Ca(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0);
    }
    
    
    /// Derivative of rate function between state: 2 and 3 for variable: V
    double der_rate_func_2_3_wrt_V(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0);
    }
    
    
    /// Derivative of rate function between state: 2 and 3 for variable: dyadic_Ca
    double der_rate_func_2_3_wrt_dyadic_Ca(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0);
    }
    
    
    /// Derivative of rate function between state: 1 and 0 for variable: V
    double der_rate_func_1_0_wrt_V(const MarkovModel& mm)
    {
      return simple::temp_scale()*(-simple::get_parameter(5)*exp(35.0/3.0 - mm._unsafe_variable(0)/3)/3);
    }
    
    
    /// Derivative of rate function between state: 1 and 0 for variable: dyadic_Ca
    double der_rate_func_1_0_wrt_dyadic_Ca(const MarkovModel& mm)
    {
      return simple::temp_scale()*(0);
    }
    
    
    /// Init params
    std::vector<double> params()
    {
      // vector which will be returned by value
      std::vector<double> params_;
      params_.push_back(1.800000);
      params_.push_back(37.000000);
      params_.push_back(6.000000);
      params_.push_back(1.000000);
      params_.push_back(2.000000);
      params_.push_back(3.000000);
      params_.push_back(4.000000);
      params_.push_back(5.000000);

      // return by value
      return params_;
    }

    /// Init param_map
    const std::map<std::string, uint> param_map()
    {
      // map which will be returned by value
      std::map<std::string, uint> param_map_;
      param_map_["Q10"] = 0;
      param_map_["T_ref"] = 1;
      param_map_["a"] = 2;
      param_map_["alpha_0"] = 3;
      param_map_["alpha_1"] = 4;
      param_map_["beta_0"] = 5;
      param_map_["beta_1"] = 6;
      param_map_["gamma_0"] = 7;

      // return by value
      return param_map_;
    }

    /// Init transition_map
    const std::vector<std::vector<uint> > transition_map()
    {
      // vector which will be returned by value
      std::vector<std::vector<uint> > transition_map_;
      
      // Transition map from state 0
      std::vector<uint> transition_map_0;
      transition_map_0.push_back(1);
      transition_map_.push_back(transition_map_0);
      
      // Transition map from state 1
      std::vector<uint> transition_map_1;
      transition_map_1.push_back(0);
      transition_map_1.push_back(2);
      transition_map_.push_back(transition_map_1);
      
      // Transition map from state 2
      std::vector<uint> transition_map_2;
      transition_map_2.push_back(1);
      transition_map_2.push_back(3);
      transition_map_.push_back(transition_map_2);
      
      // Transition map from state 3
      std::vector<uint> transition_map_3;
      transition_map_3.push_back(2);
      transition_map_.push_back(transition_map_3);

      // return by value
      return transition_map_;
    }

    /// Init rate_functions
    const std::vector<std::vector<rate_call> > rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<rate_call> > rate_functions_;
      
      // Rate functions from state 0
      std::vector<rate_call> rate_functions_0;
      rate_functions_0.push_back(rate_func_0_1);
      rate_functions_.push_back(rate_functions_0);
      
      // Rate functions from state 1
      std::vector<rate_call> rate_functions_1;
      rate_functions_1.push_back(rate_func_1_0);
      rate_functions_1.push_back(rate_func_1_2);
      rate_functions_.push_back(rate_functions_1);
      
      // Rate functions from state 2
      std::vector<rate_call> rate_functions_2;
      rate_functions_2.push_back(rate_func_2_1);
      rate_functions_2.push_back(rate_func_2_3);
      rate_functions_.push_back(rate_functions_2);
      
      // Rate functions from state 3
      std::vector<rate_call> rate_functions_3;
      rate_functions_3.push_back(rate_func_3_2);
      rate_functions_.push_back(rate_functions_3);

      // return by value
      return rate_functions_;
    }
    
    /// Init derivative of rate_functions
    const std::vector<std::vector<std::vector<rate_call> > > der_rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<std::vector<rate_call> > > der_rate_functions_;
      
      // Derivative of rate functions from state 0
      std::vector<std::vector<rate_call> > der_rate_functions_0;
      
      // Derivative of rate functions from state 0 to state 1
      std::vector<rate_call> der_rate_functions_0_1;
      der_rate_functions_0_1.push_back(der_rate_func_0_1_wrt_V);
      der_rate_functions_0_1.push_back(der_rate_func_0_1_wrt_dyadic_Ca);
      der_rate_functions_0.push_back(der_rate_functions_0_1);
      
      der_rate_functions_.push_back(der_rate_functions_0);
      
      // Derivative of rate functions from state 1
      std::vector<std::vector<rate_call> > der_rate_functions_1;
      
      // Derivative of rate functions from state 1 to state 0
      std::vector<rate_call> der_rate_functions_1_0;
      der_rate_functions_1_0.push_back(der_rate_func_1_0_wrt_V);
      der_rate_functions_1_0.push_back(der_rate_func_1_0_wrt_dyadic_Ca);
      der_rate_functions_1.push_back(der_rate_functions_1_0);
      
      // Derivative of rate functions from state 1 to state 2
      std::vector<rate_call> der_rate_functions_1_2;
      der_rate_functions_1_2.push_back(der_rate_func_1_2_wrt_V);
      der_rate_functions_1_2.push_back(der_rate_func_1_2_wrt_dyadic_Ca);
      der_rate_functions_1.push_back(der_rate_functions_1_2);
      
      der_rate_functions_.push_back(der_rate_functions_1);
      
      // Derivative of rate functions from state 2
      std::vector<std::vector<rate_call> > der_rate_functions_2;
      
      // Derivative of rate functions from state 2 to state 1
      std::vector<rate_call> der_rate_functions_2_1;
      der_rate_functions_2_1.push_back(der_rate_func_2_1_wrt_V);
      der_rate_functions_2_1.push_back(der_rate_func_2_1_wrt_dyadic_Ca);
      der_rate_functions_2.push_back(der_rate_functions_2_1);
      
      // Derivative of rate functions from state 2 to state 3
      std::vector<rate_call> der_rate_functions_2_3;
      der_rate_functions_2_3.push_back(der_rate_func_2_3_wrt_V);
      der_rate_functions_2_3.push_back(der_rate_func_2_3_wrt_dyadic_Ca);
      der_rate_functions_2.push_back(der_rate_functions_2_3);
      
      der_rate_functions_.push_back(der_rate_functions_2);
      
      // Derivative of rate functions from state 3
      std::vector<std::vector<rate_call> > der_rate_functions_3;
      
      // Derivative of rate functions from state 3 to state 2
      std::vector<rate_call> der_rate_functions_3_2;
      der_rate_functions_3_2.push_back(der_rate_func_3_2_wrt_V);
      der_rate_functions_3_2.push_back(der_rate_func_3_2_wrt_dyadic_Ca);
      der_rate_functions_3.push_back(der_rate_functions_3_2);
      
      der_rate_functions_.push_back(der_rate_functions_3);

      // return by value
      return der_rate_functions_;
    }
    
  }

  /// Initialize static variables
  double simple::_temp_scale = 1.0;
  std::vector<double> simple::_params = Initsimple::params();
  const std::map<std::string, uint> simple::_param_map = Initsimple::param_map();
  const std::vector<std::vector<uint> > simple::_transition_map = Initsimple::transition_map();
  const std::vector<std::vector<rate_call> > simple::_rate_functions = Initsimple::rate_functions();
  const std::vector<std::vector<std::vector<rate_call> > > simple::_rate_derivative_functions = Initsimple::der_rate_functions();

}

#endif
