# A simple test Markokov model of four states
#
#      0  -  1  -  2  -  3
#

from gillstep import *
from gillstep.codegeneration import generate_model_code

simple = ModelEditor("simple")

# Declarations
simple.set_parameters(
    alpha_0 = 1.0, alpha_1 = 2.0,
    beta_0  = 3.0, beta_1  = 4.0,
    gamma_0 = 5.0, a       = 6.0)

simple.set_variables("V", "dyadic_Ca")

simple.set_Q10(1.8, 37)

# Rate functions
alpha = exp(-(20-V/(R*T))/10)
beta  = exp(-(V-35)/3)
gamma = gamma_0*dyadic_Ca

simple.rates[0,1] = 2*alpha_0*alpha
simple.rates[1,2] = alpha_1*alpha
simple.rates[1,0] = beta_0*beta
simple.rates[2,1] = 2*beta_1*beta
simple.rates[3,2] = gamma
simple.rates[2,3] = a**2

# State values
simple.state_values[3] = 1

# Instantiate the GillespieStepper
gillstepper = GillespieStepper()

# JIT compile the markov model
mm, mm2 = jit(simple, gillstepper, 2)

print "Value: ", mm.value()
print "Value: ", mm2.value()
print "T_ref: ", mm.get_parameter("T_ref")
print "alpha_0: ", mm.get_parameter("alpha_0")
print "gamma_0: ", mm.get_parameter("gamma_0")
  
