// Copyright (C) 2010 Johan Hake
// Licensed under the GNU LGPL Version 3.0
//
// First added:  2010-08-23
// Last changed: 2010-08-23

#include <boost/bind.hpp>

#include "DiscreteValueReceiver.h"
#include "GillespieStepper.h"

using namespace gillstep;

//-----------------------------------------------------------------------------
DiscreteValueReceiver::DiscreteValueReceiver(GillespieStepper& stepper): 
                                             _reserved(CHUNK), _stepper(&stepper), 
					     _times(1, 0.0), _num_open(1, 0)
{
  // Bind the _receive method to the public receive attribute
  receive = boost::bind(&DiscreteValueReceiver::_receive, this, _1);
  _times.reserve(CHUNK);
  _num_open.reserve(CHUNK);
  
}
//-----------------------------------------------------------------------------
void DiscreteValueReceiver::_receive(double value)
{
  // Check size of reserved data
  // FIXME: Is this needed. Doesn't std::vector do this?
  if (_num_open.size() == _reserved){
    _reserved += CHUNK;
    _times.reserve(_reserved);
    _num_open.reserve(_reserved);
  }
  
  // Register time
  if (_stepper->time() == 0.0)
    _num_open[0] += (value == 1.0 ? 1 : 0);
  else {
    _times.push_back(_stepper->time());

    // Register the value
    if (value == 1.0)
      _num_open.push_back(_num_open[_num_open.size()-1]+1);
    else
      _num_open.push_back(_num_open[_num_open.size()-1]-1);
  }

}
//-----------------------------------------------------------------------------
const std::vector<double>& DiscreteValueReceiver::get_times() const
{
  // Return the times
  return _times;
}
//-----------------------------------------------------------------------------
const std::vector<gillstep::uint>& DiscreteValueReceiver::get_num_open() const
{
  // Return the num_open
  return _num_open;
}
//-----------------------------------------------------------------------------

