// Copyright (C) 2010 Johan Hake
// Licensed under the GNU LGPL Version 3.0
//
// First added:  2010-08-23
// Last changed: 2010-08-23

#ifndef __DISCRETEVALUERECEIVER_H
#define __DISCRETEVALUERECEIVER_H

#include <vector>
#include "common.h"

namespace gillstep
{

  // Forward declarations
  class GillespieStepper;

  // A class that can be used to collect discrete events of 0 and 1.
  class DiscreteValueReceiver
  {
  public:
    
    // Default constructor
    DiscreteValueReceiver(GillespieStepper& stepper);

    // "Protected" _receive method
    void _receive(double value);

    // Return the times
    const std::vector<double>& get_times() const;

    // Return the num open
    const std::vector<uint>& get_num_open() const;

  // Private attributes
  private:
    
    // The reserved amount of memory
    uint _reserved;

    // The gillespi stepper
    GillespieStepper* _stepper;

    // Keeping record of the discrete times
    std::vector<double> _times;

    // Keeping record of the discrete values
    std::vector<uint> _num_open;
  
  // Public attribute
  public:

    // The value receiver
    ValueReceiver receive;

  };


}


#endif


