// Copyright (C) 2010 Johan Hake.
// Licensed under the GNU LGPL Version 3.0
//
// First added:  2010-01-20
// Last changed: 2011-03-01
//
// This file provides typedefs for basic types

#ifndef __GILLSTEP_TYPES_H
#define __GILLSTEP_TYPES_H

#include <cmath>
#include <boost/function.hpp>

namespace gillstep
{

  /// Unsigned integers
  typedef unsigned int uint;

  /// Unsigned integer type, at least 32 bits
  typedef unsigned long uint32;

  /// A boost function type for receiving values
  typedef boost::function<void (double)> ValueReceiver;

  /// A constant defining size of a chunk
  const uint CHUNK = 10000;

  /// A constant defining size of machine presiton
#define  GILLSTEP_EPS  1.0e-13

  /// A constant defining size of huge number
#define  GILLSTEP_HUGE  1.0e15

  /// Helper function which returns true if the argument is close to zero
  inline bool is_zero(double arg) {return arg == 0.0 || std::fabs(arg) < GILLSTEP_EPS;}

}

#endif
