/* -*- C -*- */
// Copyright (C) 2010 Johan Hake
// Licensed under the GNU LGPL Version 2.1.
//
// First added:  2010-08-23
// Last changed: 2012-04-20
//
// ===========================================================================
// SWIG directives for GillStep
//
// The directives in this file are applied _before_ the header files has been 
// loaded.
// ===========================================================================

%include "gillstep/swig/typemaps.i"
%include "gillstep/swig/exceptions.i"
%include "gillstep/swig/docstrings.i"
 //%ignore  gillstep::DiscreteValueStepper::connect;
 //%ignore  gillstep::MarkovModel::connect;
%ignore  gillstep::GillespieStepper(uint32* const, uint32 const);
%ignore  gillstep::GillespieStepper(MTRandomGenerator&);
%ignore  gillstep::GillespieStepper::random_transition;
%ignore  gillstep::GillespieStepper::sum;
%ignore  gillstep::GillespieStepper::cumsum;
%ignore  gillstep::rate_call;

//-----------------------------------------------------------------------------
// Create a dummy class of ValueReceiver so a Proxy class of boost::function
// is created
//-----------------------------------------------------------------------------
namespace gillstep
{

  class ValueReceiver {
  public:
    ValueReceiver();
  };

}

//-----------------------------------------------------------------------------
// Create wrappers for Python functions so they can be used to bind to a 
// ValueReceiver
//-----------------------------------------------------------------------------
%{
void _call_python_function_slot(PyObject* slot, double value)
{
  // Convert the value
  PyObject* py_args = Py_BuildValue("(d)", value);
  
  // Call the python slot
  //PyObject* py_result = PyObject_CallObject(slot, py_args);
  PyObject_CallObject(slot, py_args);

  // Reduce refcount
  Py_DECREF(py_args);
  //Py_DECREF(py_result);

}

void _call_python_method_slot(PyObject* slot_object, char* method_name, double value)
{
  // Call the python slot
  PyObject* py_result = PyObject_CallMethod(slot_object, method_name, const_cast<char*>("(d)"), value);
  
  // Reduce refcount
  Py_DECREF(py_result);

}
%}

%inline%{
gillstep::ValueReceiver _bind_function(PyObject* py_slot)
{
  gillstep::ValueReceiver slot;

  // Bind the python slot to the call_python_function and assign this to 
  // the ValueReciever
  slot = boost::bind(&_call_python_function_slot, py_slot, _1);

  // Return slot
  return slot;
}

gillstep::ValueReceiver _bind_method(PyObject* py_slot_object, char* method_name)
{
  gillstep::ValueReceiver slot;

  // Bind the python slot to the call_python_method and assign this to 
  // the ValueReciever
  slot = boost::bind(&_call_python_method_slot, py_slot_object, method_name, _1);

  // Return None
  return slot;
}
%}

//-----------------------------------------------------------------------------
// User macro for expanding the connect method 
//-----------------------------------------------------------------------------
%define CONNECT_WRAPPER(TYPE)

%pythonprepend gillstep::TYPE::connect%{
    if len(args) != 1:
        raise TypeError, "expected 1 argument"
    if not hasattr(self, "_connections"):
        self._connections = []
    if not isinstance(args[0], ValueReceiver):
        import types
        slot = args[0]
        self._connections.append(slot)
        if isinstance(slot, types.FunctionType):
            slot = _bind_function(slot)
        elif isinstance(slot, types.MethodType):
            slot = _bind_method(slot.im_self, slot.im_func.__name__)
        else:
            raise TypeError, "expected a ValueReciever, Function or Method as argument"
        args = (slot,)
    else:
        self._connections.append(args[0])

%}

%enddef

CONNECT_WRAPPER(MarkovModel)
CONNECT_WRAPPER(DiscreteValueStepper)

