__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-01-26 -- 2010-09-08"
__license__  = "GNU LGPL Version 3.0 or later"

# Import names
from gillstep.modeleditor import ModelEditor
from gillstep.namespace   import gillstep_namespace
from gillstep.runtime     import update_runtime_namespace
from gillstep.loadmodel   import load_model, load_submodel

# Import submodules
import gillstep.parameterdict  as parameterdict
import gillstep.compilemodule  as compilemodule
import gillstep.logger         as logger
import gillstep.codegeneration as codegeneration
import gillstep.commands       as commands
import gillstep.config         as config
import gillstep.version        as version

# Import sympy hack
from gillstep.sympyhack import sympy 

# Import compiled extension module if any
from gillstep.cpp_import  import cpp

# Assign the automatically generated version string
__version__ = version.version
del version

_ns = gillstep_namespace()

globals().update(_ns)

__all__ = ["ModelEditor", "load_model", "load_submodel"] + _ns.keys()

# Use interactive mode by default
logger.set_raise_error(False)

# If the user has compiled gillstep with extension modules import the compiled classes
if cpp:
    cpp_names = ["get_T", "set_T", "T_abs", "GillespieStepper", "is_zero",
                 "MarkovModel", "DiscreteValueReceiver", "DiscreteValueStepper"]
    from gillstep.compilemodule import jit
    for name in cpp_names:
        exec("%s = cpp.%s"%(name, name))

    __all__.extend(cpp_names)
    __all__.append("jit")

