"Code snippets for code generation"

__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-01-26 -- 2012-05-02"
__license__  = "GNU LGPL Version 3.0 or later"

model_snippet = r"""
  /// Markov model 
  class %(class_name)s : public MarkovModel
  {
%(submodel_friends)s
  
  public:

    /// Default constructor
    %(class_name)s(GillespieStepper& gillstepper) :
        MarkovModel("%(class_name)s", gillstepper),
        _present_substep(0.0),
        _variables_05(%(num_variables)d),
        _rate_05(%(max_transitions)d),
        _rate_deriviatives_05(%(max_transitions)d)
        %(submodels_cstr_1)s
    {
      /// Resize field
      _variables_0.resize(%(num_variables)d);
      _variables_1.resize(%(num_variables)d);
      _discrete_variables.resize(%(num_discrete_variables)d);
      _present_variables = &_variables_1;
%(submodels_cstr_2)s
%(receive_objects_init)s
    }
    
    /// SubMarkov model constructor
    %(class_name)s(MarkovModel* parent, GillespieStepper& gillstepper) :
        MarkovModel("%(class_name)s", gillstepper),
        _present_substep(0.0),
        _variables_05(%(num_variables)d),
        _rate_05(%(max_transitions)d),
        _rate_deriviatives_05(%(max_transitions)d)%(submodels_cstr_1)s
    {
      // Assign parent pointer
      _parent = parent;

      /// Resize field
      _variables_0.resize(%(num_variables)d);
      _variables_1.resize(%(num_variables)d);
      _discrete_variables.resize(%(num_discrete_variables)d);
      _present_variables = &_variables_1;
%(submodels_cstr_2)s
%(receive_objects_init)s

      // Check if model can receive "parent"
      if (can_receive(parent->name()))
      {
        parent->connect(receive(parent->name()));
        _receive_parent = true;
      }
    
    }
    
    /// Destructor
    virtual ~%(class_name)s(){};
    
    /// Returns the maximal transitions from a state
    uint max_transitions() const
    {
       return %(max_transitions)d;
    }

    /// Return the 'value' of the Markov model the state represent 
    double value() const
    {
      switch (_state)
      {
%(state_values)s
      default:
        return 0.0;
      }
    
    }

    /// Return the size of the substep. 1.0 if no substep is needed
    double substep_size(double present_substep)
    {
      assert(present_substep >= 0.0);
        
      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep)
      {
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      double next_substep = 1;
      for (uint trans = 0; trans < _transition_map[_state].size(); trans++)
      {
        if (std::fabs(_rate_deriviatives_05[trans]) > 1e-10)
            next_substep = std::min(next_substep, std::fabs(
                           _gillstepper->get_max_fractional_rate_change()*
                           (_rate_05[trans]/_rate_deriviatives_05[trans])));
      }
      
      // Update the the next substep result with the present one
      next_substep = present_substep + next_substep;

      // Return next substep. Return 1.0 if it is larger than 1.0
      return next_substep > 1.0 ? 1.0 : next_substep;
    }

    /// Evaluate rate function corresponding to passed transition
    double evaluate_rate(uint transition) const
    {
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=%(class_name)s::_transition_map[_state].size())
        return 0.0;

      // Evaluate the rate
      return %(class_name)s::_rate_functions[_state][transition](*this);
    }
    
    /// Evaluate rate function corresponding to passed state and transition
    double evaluate_rate(uint from_state, uint to_state) const
    {
      switch (from_state)
      {
%(evaluate_rate)s
      default:
        return 0.0;
      }
    }
    
    /// Evaluate rate function corresponding to passed transition for a substep
    double evaluate_substep_rate(uint transition, double present_substep,
                                 double next_substep)
    {
      assert(present_substep>=0);
      assert(next_substep>=present_substep);
      
      // Return 0.0 if the present state does support the passed transition
      if (transition<0||transition>=%(class_name)s::_transition_map[_state].size())
        return 0.0;

      // Check if the variables have changed
      if (_retabulate_derivatives || present_substep != _present_substep){
        _present_substep = present_substep;
        _tabulate_rate_deriviatives();
      }

      // Use tabulated values
      return _rate_05[transition] + (next_substep-_present_substep)*
        (_rate_deriviatives_05[transition]);
    }
    
    /// Change the state to the given transition
    bool change_state(int transition)
    {
      /// If forced to not change the value of this Markov Model
      if (fix_state)
        return false;
        
      /// We need to retabulate derivatives
      _retabulate_derivatives = true;
        
      const int num_transitions = %(class_name)s::_transition_map[_state].size();
      assert(transition<num_transitions);
        
      /// Update the state from the transition
      if (transition>=0)
        _state = %(class_name)s::_transition_map[_state][transition];

      /// If this is a SubMarkov model return the parents change_state value
      if (_parent)
        return _parent->change_state(-1);

      /// If this is not a SubMarkov model check if my value has change
      const bool changed = _old_value != value();
      _old_value = value();
      return changed;
    }

    /// Get temperature scale
    static inline double temp_scale()
    {
      return %(class_name)s::_temp_scale;
    }

    /// Get parameter
    static inline double get_parameter(uint ind)
    {
      return %(class_name)s::_params[ind];
    }

    /// Get parameter
    double get_parameter(const std::string& name) const
    {
      return _params[_find_parameter(name)];
    }
    
    /// Set parameter
    void set_parameter(const std::string& name, double value)
    {
      //std::cout << "Setting " << name << " to " << value << std::endl;
      _params[_find_parameter(name)] = value;
      // If the temperature scale is change update it
      if (name == "T_ref" or name == "Q10")
        update_temp_scale();
    }

    /// Get parameter names
    virtual std::vector<std::string> get_parameter_names() const
    {
      std::vector<std::string> parameter_names;

%(parameter_names)s

      // Return by value
      return parameter_names;
    };

    /// Return the number of states
    inline uint num_states() const
    {
      return %(num_states)d;
    }

    /// Update the temperature scale factor
    void update_temp_scale()
    {
      %(class_name)s::_temp_scale = pow(%(class_name)s::_params[0], ((get_T()-T_abs())-%(class_name)s::_params[1])/10);
    }

  private:

    /// Tabulate rate derivatives
    void _tabulate_rate_deriviatives()
    {
      // Local variables
      uint trans, var;

      // Update variables at present substep
      for (var=0; var < _variables_05.size(); var++)
      {
        _variables_05[var] = _variables_1[var]*_present_substep +
                             (1-_present_substep)*_variables_0[var];
        //std::cout << "Variable: " << _variables_05[var] << std::endl;
      }
        
      // Set the present variables to present interpolated values
      _present_variables = &_variables_05;

      for (trans=0; trans < _transition_map[_state].size(); trans++)
      {
        _rate_05[trans] = std::max(0.1, %(class_name)s::_rate_functions
                                   [_state][trans](*this));
        //std::cout << "Rate: " << _rate_05[var] << std::endl;
        _rate_deriviatives_05[trans] = 0.0;
        for (var=0; var<_variables_0.size(); var++){
          _rate_deriviatives_05[trans] += %(class_name)s::_rate_derivative_functions
                                         [_state][trans][var](*this)*
                                         (_variables_1[var] - _variables_05[var]);
        //std::cout << "Rate der: " << _rate_deriviatives_05[var] << std::endl;
        }
      }

      // Set the present variables to the last received values
      _present_variables = &_variables_1;

      // Set flag preventing unessesary tabulation
      _retabulate_derivatives = false;
    }

    /// Check parameter name
    static uint _find_parameter(const std::string& name)                              
    {
      std::map<std::string, uint>::const_iterator it = %(class_name)s::_param_map.find(name);
      if (it == %(class_name)s::_param_map.end())
        throw std::runtime_error("\"" + name + "\" is not a parameter in this Markov Model");                         
      return it->second;
    }

%(receive_methods)s

    /// Data members

    // Keep track of which substep we are on
    double _present_substep;

    // Variables values inside time step
    std::vector<double> _variables_05;

    /// The interpolated rate values and its derivatives wrt to the variables
    std::vector<double> _rate_05;
    std::vector<double> _rate_deriviatives_05;

%(submodel_data)s
    /// Temperature scale factor
    static double _temp_scale;

    /// An array for the parameter values
    static std::vector<double> _params;

    /// Mapping names to parameter indices
    static const std::map<std::string, uint> _param_map;
    
    /// Mapping state and transition number to state
    static const std::vector<std::vector<uint> > _transition_map;

    /// Mapping state and transition number to rate function
    static const std::vector<std::vector<rate_call> > _rate_functions;

    /// Mapping state, transition number and continuous variables to rate
    /// derivative function
    static const std::vector<std::vector<std::vector<rate_call> > > _rate_derivative_functions;
    
  };

  // Name space for hiding static initalization functions
  namespace Init%(class_name)s
  {

    /// Define Rate functions
%(define_rate_functions)s

    /// Define derivatives of Rate functions
%(define_derivatives_of_rate_functions)s
    /// Init params
    std::vector<double> params()
    {
      // vector which will be returned by value
      std::vector<double> params_;
%(init_params)s

      // return by value
      return params_;
    }

    /// Init param_map
    const std::map<std::string, uint> param_map()
    {
      // map which will be returned by value
      std::map<std::string, uint> param_map_;
%(init_param_map)s

      // return by value
      return param_map_;
    }

    /// Init transition_map
    const std::vector<std::vector<uint> > transition_map()
    {
      // vector which will be returned by value
      std::vector<std::vector<uint> > transition_map_;
%(init_transition_map)s

      // return by value
      return transition_map_;
    }

    /// Init rate_functions
    const std::vector<std::vector<rate_call> > rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<rate_call> > rate_functions_;
%(init_rate_functions)s

      // return by value
      return rate_functions_;
    }
    
    /// Init derivative of rate_functions
    const std::vector<std::vector<std::vector<rate_call> > > der_rate_functions()
    {
      // vector which will be returned by value
      std::vector<std::vector<std::vector<rate_call> > > der_rate_functions_;
%(init_derivative_of_rate_functions)s

      // return by value
      return der_rate_functions_;
    }
    
  }

  /// Initialize static variables
  double %(class_name)s::_temp_scale = 1.0;
  std::vector<double> %(class_name)s::_params = Init%(class_name)s::params();
  const std::map<std::string, uint> %(class_name)s::_param_map = Init%(class_name)s::param_map();
  const std::vector<std::vector<uint> > %(class_name)s::_transition_map = Init%(class_name)s::transition_map();
  const std::vector<std::vector<rate_call> > %(class_name)s::_rate_functions = Init%(class_name)s::rate_functions();
  const std::vector<std::vector<std::vector<rate_call> > > %(class_name)s::_rate_derivative_functions = Init%(class_name)s::der_rate_functions();
"""

model_form = {}.fromkeys(["submodel_friends","class_name", "init_param_map"\
"max_transitions","init_params", "init_rate_functions", "state_values",\
"init_derivative_of_rate_functions", "init_transition_map",\
"define_rate_functions", "define_derivatives_of_rate_functions",\
"num_states", "num_parameters", "parameter_names", "receive_methods",\
"num_discrete_variables", "num_variables", "submodels_cstr_1",\
"submodels_cstr_2", "evaluate_rate", "receive_objects_init"],
"NOT_ASSIGNED")

file_snippet = r"""
// gillstep generated Markov model code

#ifndef __%(CLASS_NAME)s_H
#define __%(CLASS_NAME)s_H

#include <vector>
#include <map>
#include <cmath>
#include <cassert>
#include <stdexcept>
#include <algorithm>
#include <boost/bind.hpp>

#include <gillstep/common.h>
#include <gillstep/MarkovModel.h>
#include <gillstep/GillespieStepper.h>
#include <gillstep/temperature.h>
%(includes)s
namespace gillstep
{
%(model_code)s
}

#endif
"""

file_form = {}.fromkeys(["CLASS_NAME", "model_code", "includes"],"NOT_ASSIGNED")
