__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-01-26 -- 2012-02-22"
__license__  = "GNU LGPL Version 3.0 or later"

__all__ = ["gillstep_namespace"]

# Import statements
import sympy
import sympy.functions
from sympy.core import relational
from gillstep.gillstepsymbol import GillstepSymbol

class Conditional(sympy.functions.Piecewise):
    def __new__(cls, cond, true_value, false_value):
        return sympy.functions.Piecewise((true_value, cond), (false_value, True))

# Import gillstep modules
import gillstep.constants

T = GillstepSymbol("T")

def gillstep_namespace():

    # Create the gillstep namespace
    ns = dict()
    
    # Add the sympy functions to the namespace
    ns.update(sympy.functions.__dict__)
    
    # Add the sympy relationals to the namespace
    ns.update(relational.__dict__)
    
    # Add the gillstep.constants to the namespace
    ns.update(gillstep.constants.__dict__)

    # Add the temperature to the namespace
    ns["T"] = T

    # Add the Conditional sub class of Piecwise
    ns["Conditional"] = Conditional

    # Remove polluted names
    for n in ns.keys():
        if n[:2] == "__" and n[-2:] == "__":
            ns.pop(n)

    return ns
