__author__ = "Johan Hake (hake.dev@gmail.com)"
__copyright__ = "Copyright (C) 2010 " + __author__
__date__ = "2010-08-18 -- 2012-07-20"
__license__  = "GNU LGPL Version 3.0 or later"

# A fix for crappy code printing for conditionals

__all__ = ['sympy']

from distutils.version import LooseVersion

try:
    import sympy
except Exception, e:
    print "Problem loading sympy."
    print e
    exit(1)
    
from sympy.core import S, C
from sympy.printing.ccode import CCodePrinter
from sympy.printing.precedence import precedence
from sympy.printing import StrPrinter
from types import MethodType

#if LooseVersion(sympy.__version__) < "0.7.1":
#    raise ImportError("Expected sympy version 0.7.1")

# Better output to c for conditionals
def my_print_Piecewise(self, expr):
    result = ""
    for e, c in expr.args[:-1]:
        result += "(%s ? %s : "%(self._print(c), self._print(e))
    last_line = "%s)" % self._print(expr.args[-1].expr)
    return result+last_line

def my_print_Function(self, expr):
    #print expr.func.__name__, expr.args
    if expr.func.__name__ == "Max":
        return "std::max(%s)" % self.stringify(expr.args, ", ")
    elif expr.func.__name__ == "Min":
        return "std::min(%s)" % self.stringify(expr.args, ", ")
    elif expr.func.__name__ == "ceiling":
        return "std::ceil(%s)" % self.stringify(expr.args, ", ")
    elif expr.func.__name__ == "Abs" and not expr.args[0].is_integer:
        return "std::fabs(%s)" % self.stringify(expr.args, ", ")
    else:
        return "std::" + expr.func.__name__.lower() + \
               "(%s)"%self.stringify(expr.args, ", ")

def my_print_Pow(self, expr):
    PREC = precedence(expr)
    if expr.exp is S.NegativeOne:
        return '1.0/%s'%(self.parenthesize(expr.base, PREC))
    elif expr.exp.is_integer:
        return "(%s)" % ("*".join(self._print(expr.base) \
                                  for i in xrange(int(expr.exp))))
    elif expr.exp == 0.5:
        return 'std::sqrt(%s)' % self._print(expr.base)
    else:
        return 'std::pow(%s, %s)'%(self._print(expr.base),
                              self._print(expr.exp))

# Overwrite the crappy _print_Piecwise command in sympy
CCodePrinter._print_Piecewise = MethodType(my_print_Piecewise,
                                           None, CCodePrinter)

CCodePrinter._print_Pow = MethodType(my_print_Pow,
                                     None, CCodePrinter)
CCodePrinter._print_Function = MethodType(my_print_Function,
                                           None, CCodePrinter)

