"""Run all demos."""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__date__ = "2010-03-19 -- 2010-10-15"
__copyright__ = "Copyright (C) 2010 " + __author__
__license__  = "GNU LGPL version 3.0"

from time import time

from gillstep.commands import get_status_output
import re, sys, os, platform

# Location of all demos
demodir = os.path.join(os.curdir, "..", "..", "demo")

# Demos to run
cppdemos = []
pydemos = []
for dpath, dnames, fnames in os.walk(demodir):
    if os.path.basename(dpath) == 'cpp':
        if os.path.isfile(os.path.join(dpath, 'SConstruct')):
            cppdemos.append(dpath)
    elif os.path.basename(dpath) == 'python':
        if os.path.isfile(os.path.join(dpath, 'demo.py')):
            pydemos.append(dpath)

print "Running all demos"
print ""
print "Found %d C++ demos" % len(cppdemos)
print "Found %d Python demos" % len(pydemos)
print ""
import pprint

failed = []
timing = []

# Run C++ demos
for demo in cppdemos:
    print "----------------------------------------------------------------------"
    print "Running C++ demo %s" % (demo)
    print ""
    cppdemo_ext = ''
    if platform.system() == 'Windows':
        cppdemo_ext = '.exe'
    cppdemo_prefix = demo.split(os.path.sep)[-2] # Do not use this...
    if os.path.isfile(os.path.join(demo, 'demo' + cppdemo_ext)):
        t1 = time()
        output = get_status_output("cd %s && .%sdemo%s" % \
                                     (demo, os.path.sep, cppdemo_ext))
        t2 = time()
        timing += [(t2 - t1, demo)]
        if output[0] == 0:
            print "OK"
        elif output[0] == 10: # Failing but exiting gracefully
            print "ok (graceful exit on fail)"
        else:
            print "*** Failed"
            print output[1]
            failed += [(demo, "C++", output[1])]
    else:
        print "*** Warning: missing demo"

# Run Python demos
for demo in pydemos:
    print "----------------------------------------------------------------------"
    print "Running Python demo %s" % (demo)
    print ""
    if os.path.isfile(os.path.join(demo, 'demo.py')):
        t1 = time()
        output = get_status_output("cd %s && python demo.py" % (demo))
        t2 = time()
        timing += [(t2 - t1, demo)]
        if output[0] == 0:
            print "OK"
        elif output[0] == 10: # Failing but exiting gracefully
            print "ok (graceful exit on fail)"
        else:
            print "*** Failed"
            print output[1]
            failed += [(demo, "Python", output[1])]
    else:
        print "*** Warning: missing demo"

# Print summary of time to run demos
timing.sort()
print ""
print "Time to run demos:"
print "\n".join("%.2fs: %s" % t for t in timing)

total_no_demos = len(pydemos) + len(cppdemos)

# Print output for failed tests
print ""
if len(failed) > 0:
    print "%d demo(s) out of %d failed, see demo.log for details." % \
          (len(failed), total_no_demos)
    file = open("demo.log", "w")
    for (test, interface, output) in failed:
        file.write("----------------------------------------------------------------------\n")
        file.write("%s (%s)\n" % (test, interface))
        file.write("\n")
        file.write(output)
        file.write("\n")
        file.write("\n")
else:
    print "All demos checked: OK"

# Return error code if tests failed
sys.exit(len(failed) != 0)
