"""Run all unit tests."""

__author__ = "Johan Hake (hake.dev@gmail.com)"
__date__ = "2010-03-19 -- 2010-11-24"
__copyright__ = "Copyright (C) 2010 " + __author__
__license__  = "GNU LGPL version 3.0"

import sys, os, re
import platform
from gillstep.commands import get_output

# Tests to run
tests = ["gillespiestepper", "markovmodels"]

# Tests only available in Python
only_python = ["python-extras"]

# Run tests
failed = []
for test in tests:
    print ""
    print "Running unit tests for %s" % test
    print "----------------------------------------------------------------------"

    cpptest_ext = ''
    if platform.system() == 'Windows':
        cpptest_ext = '.exe'
    print "C++:   ",
    if not test in only_python:
        output = get_output("cd %s%scpp && .%stest%s" % \
                           (test, os.path.sep, os.path.sep, cpptest_ext))
        if "OK" in output:
            num_tests = int(re.search("OK \((\d+)\)", output).groups()[0])
            print "OK (%d tests)" % num_tests
        else:
            print "*** Failed"
            failed += [(test, "C++", output)]
    else:
        print "Skipping"

    print "Python:",
    output = get_output("cd %s%spython && python .%stest.py" % \
                       (test, os.path.sep, os.path.sep))
    if "OK" in output:
        num_tests = int(re.search("Ran (\d+) test", output).groups()[0])
        print "OK (%d tests)" % num_tests
    else:
        print "*** Failed"
        failed += [(test, "Python", output)]

# Print output for failed tests
for (test, interface, output) in failed:
    print "One or more unit tests failed for %s (%s):" % (test, interface)
    print output

# Return error code if tests failed
sys.exit(len(failed) != 0)
